local addonName = ...
local core = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceEvent-3.0", "AceConsole-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)

core.factions = {}
core.gainsReputation = {}
core.timerReputation = false

core.currencies = nil
core.gainsCurrency = {}
core.timerCurrency = false

core.defaultColors = FACTION_BAR_COLORS
core.prettyColors = {
   [1] = {r = 0.63, g = 0.00, b = 0.00}, -- Hated
   [2] = {r = 0.63, g = 0.00, b = 0.00}, -- Hostile
   [3] = {r = 0.63, g = 0.00, b = 0.00}, -- Unfriendly
   [4] = {r = 0.82, g = 0.67, b = 0.00}, -- Neutral
   [5] = {r = 0.50, g = 0.74, b = 0.43}, -- Friendly
   [6] = {r = 0.49, g = 0.73, b = 0.42}, -- Honored
   [7] = {r = 0.47, g = 0.68, b = 0.47}, -- Revered
   [8] = {r = 0.40, g = 0.62, b = 0.40}, -- Exalted
}

core.defaults = {
   profile  = {
      prefix = "[%H:%M:%S]",
      prefixColor = {r = 0.66, g = 0.66, b = 0.66},
      colors = true,
      skipDefaultText = true,
      changeReputationBar = true,
      delay = 0.5,
   }
}


function core:OnInitialize()
   self.db = LibStub("AceDB-3.0"):New("TrackerDB", self.defaults, true)

   self:RegisterEvent("PLAYER_ENTERING_WORLD", "WorldEnter")
   self:RegisterEvent("ADDON_LOADED", "Loaded")

   self:RegisterEvent("CURRENCY_DISPLAY_UPDATE", "CurrenciesPreChanges")

   ChatFrame_AddMessageEventFilter("CHAT_MSG_COMBAT_FACTION_CHANGE", _ChatFilter)
   ChatFrame_AddMessageEventFilter("CHAT_MSG_CURRENCY", _ChatFilter)
end

function core:Loaded()
   self:UnregisterEvent("ADDON_LOADED")
   self:updateColors()
end

function core:WorldEnter()
   self:UnregisterEvent("PLAYER_ENTERING_WORLD")
   self.factions = self:GetFactions()
   self.currencies = self:GetCurrencies()
end

function core:updateColors()
   if self.db.profile.colors then
      FACTION_BAR_COLORS = self.prettyColors
   else
      FACTION_BAR_COLORS = self.defaultColors
   end
end

function _ChatFilter(self, event, msg, ...)
   local skip = false
   if event == "CHAT_MSG_COMBAT_FACTION_CHANGE" then
      if core.db.profile.delay > 0 then
         if core.timerReputation ~= nil then
            core:CancelTimer(core.timerReputation)
         end
         core.timerReputation = core:ScheduleTimer("FactionsChanges", core.db.profile.delay)
      else
         core:FactionsChanges()
      end
      skip = core.db.profile.skipDefaultText
   elseif event == "CHAT_MSG_CURRENCY" then
      skip = core.db.profile.skipDefaultText
   end
   return skip, msg, ...
end

function core:CurrenciesPreChanges()
   if self.db.profile.delay > 0 then
      if self.timerCurrency ~= nil then
         self:CancelTimer(self.timerCurrency)
      end
      self.timerCurrency = self:ScheduleTimer("CurrenciesChanges", self.db.profile.delay)
   else
      self:CurrenciesChanges()
   end
end

function core:FactionsChanges()
   local factions = self.GetFactions()
   local old = nil
   local diff = 0

   for i, new in pairs(factions) do
      if self.gainsReputation[i] == nil then
         self.gainsReputation[i] = 0
      end
      if self.factions[i] ~= nil then
         old = self.factions[i]
         diff = new.barValue - old.barValue
         if diff ~= 0 then
            self.gainsReputation[i] = self.gainsReputation[i] + diff
            self:PrintReputation(new, diff, false)
         end
      else
         self.gainsReputation[i] = new.barValue
         self:PrintReputation(new, new.barValue, true)
      end

   end
   self.factions = factions
end

function core:CurrenciesChanges()
   if self.currencies == nil then
      return
   end

   local currencies = self.GetCurrencies()
   local old = nil
   local diff = 0

   for i, new in pairs(currencies) do
      if self.gainsCurrency[i] == nil then
         self.gainsCurrency[i] = 0
      end
      if self.currencies[i] ~= nil then
         old = self.currencies[i]
         diff = new.currentAmount - old.currentAmount
         if diff ~= 0 then
            self.gainsCurrency[i] = self.gainsCurrency[i] + diff
            self:PrintCurrency(new, diff, false)
         end
      else
         self.gainsCurrency[i] = new.currentAmount
         self:PrintCurrency(new, new.currentAmount, true)
      end

   end
   self.currencies = currencies
end

function core:PrintReputation(new, diff, newFaction)
   local color = {
      main = "|cFF8080ff",
      off = "|cFFA7A7ff",
      points = "",
      gain = "|cFF40ff40",
      loss = "|cFFff4040"
   }

   local total = self.gainsReputation[new.factionID]

   if diff > 0 then
      color.points = color.gain
   else
      color.points = color.loss
   end

   local str = color.main .. new.name .. ": " .. color.points .. diff .. " p." .. color.main

   if not newFaction then
      str = str .. color.off .. "  (" .. L["session"] .. ": " .. total .. ", " .. L["left"] .. ": "  .. new.currentNext .. " "  .. new.intervalNextName .. ")"
   end

   self:_Print(str)

   if self.db.profile.changeReputationBar then
      SetWatchedFactionIndex(new.index)
   end
end

function core:PrintCurrency(new, diff, newCurrency)
   local color = {
      main = "|cFF00aa00",
      off = "|cFF42cf42",
      points = "",
      gain = "|cFF40ff40",
      loss = "|cFFff4040"
   }

   local total = self.gainsCurrency[new.index]

   if diff > 0 then
      color.points = color.main
   else
      color.points = color.loss
      diff =  0 - diff
   end

   local str = new.link .. color.main .. ": " .. color.points .. "x" .. diff .. "" .. color.main .. ". "


   str = str .. color.off .. "("

   str = str .. L["session"] .. ": " .. total .. ", "

   str = str .. L["total"] .. ": " .. new.currentAmount
   if new.totalMax > 0 then
      str = str .. "/" .. new.totalMax
   end

   str = str .. ")"

   -- if not newCurrency then
      -- str = str .. color.off .. "  (session: " .. total .. ", left: "  .. new.currentNext .. " "  .. new.intervalNextName .. ")"
   -- end

   self:_Print(str)
end

function core:GetFactions()
   local factions = {};

   ExpandAllFactionHeaders();

   for i = 1, GetNumFactions() do
      local
         name,
         description,
         standingId,
         barMin,
         barMax,
         barValue,
         atWarWith,
         canToggleAtWar,
         isHeader,
         isCollapsed,
         hasRep,
         isWatched,
         isChild,
         factionID,
         hasBonusRepGain,
         canBeLFGBonu
            = GetFactionInfo(i)

      if not isHeader then

         if barValue == nil then
            barValue = 0;
         end

         local standingNextId = standingId + 1
         if standingNextId > 8 then standingNextId = 8 end
         standingName = _G["FACTION_STANDING_LABEL" .. standingId];
         standingNextName = _G["FACTION_STANDING_LABEL" .. standingNextId];

         currentAmount = barValue - barMin;
         currentMax = barMax - barMin;

         local faction = {
               factionID = factionID,
               index = i,
               name = name,

               intervalId = standingId,
               intervalName= standingName,
               intervalNextId = standingNextId,
               intervalNextName = standingNextName,

               current = currentAmount,
               currentMax = currentMax,
               currentNext = currentMax - currentAmount,

               barMin = barMin,
               barMax = barMax,
               barValue = barValue
            }

         factions[factionID] = faction

      end
   end


   return factions;
end

function core:GetCurrencies()
   local currencies  = {};

   for i = 1,1500,1 do
      local name, currentAmount, texture, earnedThisWeek, weeklyMax, totalMax, isDiscovered, rarity = GetCurrencyInfo(i);

      if name ~= nil and isDiscovered then
         local currency = {
            index = i,
            name = name,
            texture = texture,
            link = GetCurrencyLink(i),
            rarity = rarity,

            currentAmount = currentAmount,

            weekAmount = earnedThisWeek,
            weeklMax = weeklyMax,

            totalMax = totalMax,

            isDiscovered = isDiscovered

         }
         currencies[i] = currency
      end
   end

   return currencies;
end

function core:_Print(...)
   local prefix = self.db.profile.prefix
   if prefix == "" then
      print(...)
      return
   end
   local color = self.db.profile.prefixColor
   local parsed = ("|cff%02x%02x%02x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, date(prefix))
   print(parsed, ...)
end
