local addonName = ...
local core = LibStub("AceAddon-3.0"):GetAddon(addonName)
local module = core:NewModule(addonName.."Config", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)

local db
local optionsFrame

local name, title, notes = GetAddOnInfo(addonName)

local options = {
   name = title,
   -- handler = addon,
   type = "group",
   get = function(info) return db[info[#info]] end,
   set = function(info, v) db[info[#info]] = v end,
   args = {
      description = {
         type = "description",
         name = notes,
         order = 0,
      },
      general = {
         type = "group",
         name = L["General"],
         order = 1,
         inline = true,
         args = {
            skipDefaultText = {
               type = "toggle",
               name = L["Skip Default Text"],
               desc = L["Skip default text from gaining reputation or currencies."],
               descStyle = "inline",
               order = 1,
               width = "full",
            },
            prefix = {
               type = "input",
               name = L["Output Prefix"],
               desc = L["You can use: %H = hour, %M = minutes, %S = seconds (empty to disable)."],
               order = 2,
               -- width = "full",
            },
            prefixColor = {
               type = "color",
               name = L["Prefix Color"],
               desc = L["Color for the prefix text"],
               order = 3,
               -- width = "full",
               get = function(info)
                  color = db[info[#info]]
                  return color.r, color.g, color.b
               end,
               set = function(info, r, g, b)
                  db[info[#info]] = {r = r, g = g, b = b}
                  core:_Print("Test!")
               end,
            },
            delay = {
               type = "range",
               name = L["Delay time (seconds)"],
               desc = L["Set the delay time to join all gains in short periods of time (0 to disable)."],
               min = 0, max = 10, step = 0.25,
               order = 4,
               width = "full",
            },
         },
      },
      reputation = {
         type = "group",
         name = L["Reputations"],
         order = 2,
         inline = true,
         args = {
            colors = {
               type = "toggle",
               name = L["Change Reputation Colors"],
               desc = L["Change reputation colors in faction list."],
               descStyle = "inline",
               order = 1,
               width = "full",
               set = function(info, v) db[info[#info]] = v; core:updateColors() end,
            },
            changeReputationBar = {
               type = "toggle",
               name = L["Change Reputation Bar"],
               desc = L["Automatic change the faction on the reputation bar."],
               descStyle = "inline",
               order = 2,
               width = "full",
            },
         },
      },
      reset = {
         type = "execute",
         name = L["Reset Options"],
         order = -1,
         func = function() core.db:ResetProfile() end,
         confirm = true,
         confirmText = L["Do you really want to reset the options?"],
      },
   },
}

function module:OnInitialize()
   db = core.db.profile

   -- options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(core.db)

   LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, options)
   optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, addonName)

end


